import regular from "./regular";
/**
 * iov内部通用函数
 */
export default {
  /**
   * 监听全局resize事件
   */
  addResizeLisener() {
    window.onresize = this.debounce(() => {
      ELEMENT.notify({
        title: "提示",
        message: "检测到您调整了窗口大小,建议刷新页面以便调整到最佳使用效果(F5)",
        type: "warning",
        duration: 3000
      });
    }, 1000);
  },
  regular,
  delete(tips, callback) {
    ELEMENT.MessageBox.confirm(tips, "提示", {
      confirmButtonText: "确定",
      cancelButtonText: "取消",
      type: "warning",
      closeOnClickModal: false
    })
      .then(() => {
        if (callback) {
          callback();
        }
      })
      .catch(() => {});
  },
  /**
   * 查重校验
   */
  unique(fun, id = null, key = "name", text = "该名称已被使用", extra) {
    return this.debounce((rule, value, callback) => {
      if (value) {
        const data = {};
        data[key] = value;
        if (extra) {
          for (const x in extra) {
            data[x] = extra[x];
            if (extra[x] == "") {
              callback();
            }
          }
        }
        fun(data).then(res => {
          // 编辑
          if (id) {
            if (res.content.length > 0 && id != res.content[0].id) {
              callback(new Error(text));
            } else {
              callback();
            }
          }
          // 新建
          else if (res.content.length > 0) {
            callback(new Error(text));
          } else {
            callback();
          }
        });
      }
    }, 1000);
  },
  /**
   * 对于非字符值控件必填项校验
   */
  validateRequire(msg) {
    return (rule, value, callback) => {
      let flag = false;
      if (this.isArray(value)) {
        flag = value.length == 0;
      } else if (this.isString(value)) {
        flag = value == "";
      } else {
        flag = !value;
      }
      if (!flag) {
        callback();
      } else {
        callback(new Error(msg));
      }
    };
  },
  getRandomWord(num) {
    const arr = [
      "A",
      "B",
      "C",
      "D",
      "E",
      "F",
      "G",
      "H",
      "I",
      "J",
      "K",
      "L",
      "M",
      "N",
      "O",
      "P",
      "Q",
      "R",
      "S",
      "T",
      "U",
      "V",
      "W",
      "X",
      "Y",
      "Z"
    ];
    if (undefined !== num && num !== null) {
      let idvalue = "";
      for (let i = 0; i < num; i++) {
        idvalue += arr[Math.floor(Math.random() * 26)];
      }
      return idvalue;
    }
  },
  // 表单处理公共方法
  // 递归调获取表单的值
  getValidItems(nodes, result = []) {
    return nodes.reduce((current, item) => {
      if (item.hasOwnProperty("value")) {
        result.push(item);
      } else if (item.children && item.children.length > 0) {
        return this.getValidItems(item.children, result);
      }
      return result;
    }, result);
  },
  /* 禁用表单元素 */
  disabledAllItems(nodes) {
    if (nodes.children && nodes.children.length > 0) {
      nodes.children.forEach(node => {
        this.disabledAllItems(node);
      });
    }
    if (nodes && nodes.props && nodes.props.disabled == false) {
      nodes.props.disabled = true;
    }
  },

  // tree型数据转一维数组
  treeToList(tree) {
    let queen = [];
    const out = [];
    queen = queen.concat(tree);
    while (queen.length) {
      const first = queen.shift();
      if (first.children) {
        queen = queen.concat(first.children);
        delete first.children;
      }

      out.push(first);
    }
    return out;
  },

  // 一维数组转tree型数据
  listToTree(list) {
    const map = {};
    let node;
    const tree = [];
    let i;
    for (i = 0; i < list.length; i++) {
      list[i].required = list[i].required == true ? "1" : "0";
      map[list[i].treeId] = list[i];
      list[i].children = [];
    }
    for (i = 0; i < list.length; i += 1) {
      node = list[i];
      if (node.treeId !== "-1") {
        map[node.treeParentId].children.push(node);
      } else {
        tree.push(node);
      }
    }
    return tree;
  },
  // 递归 disbaled
  funcDisable(data) {
    for (let i = 0; i < data.length; i++) {
      if (data[i].children) {
        if (data[i].data == "station") {
          data[i].name = `${data[i].name}(岗位)`;
          data[i].disabled = true;
        } else if (data[i].data == "department") {
          data[i].name = `${data[i].name}(部门)`;
          data[i].disabled = true;
        } else if (data[i].data == "company") {
          data[i].disabled = true;
        }
        this.funcDisable(data[i].children);
      }
    }
  },
  // json格式化
  formatJson(json, options) {
    let reg = null;
    let formatted = "";
    let pad = 0;
    const PADDING = "    ";
    options = options || {};
    options.newlineAfterColonIfBeforeBraceOrBracket =
      options.newlineAfterColonIfBeforeBraceOrBracket === true;
    options.spaceAfterColon = options.spaceAfterColon !== false;
    if (typeof json !== "string") {
      json = JSON.stringify(json);
    }
    json = JSON.parse(json);
    json = JSON.stringify(json);
    reg = /([\{\}])/g;
    json = json.replace(reg, "\r\n$1\r\n");
    reg = /([\[\]])/g;
    json = json.replace(reg, "\r\n$1\r\n");
    reg = /(\,)/g;
    json = json.replace(reg, "$1\r\n");
    reg = /(\r\n\r\n)/g;
    json = json.replace(reg, "\r\n");
    reg = /\r\n\,/g;
    json = json.replace(reg, ",");
    if (!options.newlineAfterColonIfBeforeBraceOrBracket) {
      reg = /\:\r\n\{/g;
      json = json.replace(reg, ":{");
      reg = /\:\r\n\[/g;
      json = json.replace(reg, ":[");
    }
    if (options.spaceAfterColon) {
      reg = /\:/g;
      json = json.replace(reg, ": ");
    }
    json.split("\r\n").forEach((node, index) => {
      let i = 0;
      let indent = 0;
      let padding = "";
      if (node.match(/\{$/) || node.match(/\[$/)) {
        indent = 1;
      } else if (node.match(/\}/) || node.match(/\]/)) {
        if (pad !== 0) {
          pad -= 1;
        }
      } else {
        indent = 0;
      }
      for (i = 0; i < pad; i++) {
        padding += PADDING;
      }
      formatted += `${padding + node}\r\n`;
      pad += indent;
    });
    return formatted;
  }
};
